%% This code generates IRFs to a one-standard-deviation shock given the state space representation and RI solution for the optimal signal structure 
%% in the dynamic, multivariate rational inattention (LQG) framework.
%% (Dividing by the standard deviation gives IRF to one unit innovation shock )
%% This code also generates momoments for the states and controls under RI and under full information
%% The state transition allows for endogenous controls, IRF computed based on Kalman filter and augmented state space
%% Controlled states: x_t = A*x_{t-1} + B*u_{t-1} + L*e_t, where e_t is a Gaussian white noise with ne dimensional identity covariance matrix
%% Signal: s_t = C*x_t + v_t
%% Control: u_t = -F*\hat{x_t}
%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Input:
%
% A 	     : State transition matrix of dimension nx by nx
% B          : Feedback matrix of control to state, dimension nxk. If pure exogenous states, specify B = zeros(nx,nu), nu is the dimension of endogenous variable that user want to generates IRF on
% C          : Coefficient matrix of signals in terms of states, dimension rxn
% F          : Optimal control's decision rule matrix, dimension nu x nx, certainty equivalence holds. If exogenous states, F is speficied by users as decision rule of any endogenous variable that they want to generate IRF
% Sigma      : Optimal posterior covariance matrix for x_t obtained from RI problem,dimension nxn
% V          : Covariance matrix of endogenous signal noise v_t, dimension rxr
% L          : Exogenous nx by ne matrix coefficients of orthogonal shocks 
% Period     : Number of Periods for IRF
% J          : Order of autocovariance matrix
%
%% Output:
%
% X         : Response matrix array for augmented state vector to nx structural shock and r noise shock; X_t =[x_t; x^_t]; dimension 2n x Period x (nx+r) 
% u         : Response matrix array for control variable or endogenous variable u_t to nx structural shock and r noise shock;  dimension nu x Period x (nx+r)
% u_full    : Full information response matrix array for control variable or endogenous variable u_t to ne structural shock; dimension nu x Period x ne
% X_full    : Full information response matrix array for state (nx by Period by ne)
%
% SigxJ     : E[x(t)x(t+J)'] under partial information
% SiguJ     : E[u(t)u(t+J)'] under partial information
%Sigx_fullJ : E[x(t)x(t+J)'] under full information
%Sigu_fullJ : E[u(t)u(t+J)'] under full information
%
% Written by Jianjun Miao, Boston University, email: miaoj@bu.edu
%            and Jieran Wu, Zhejiang University, email: jw5ya@zju.edu.cn
%
% When using this code, please cite 
% Miao, Jianjun and Jieran Wu, 2020, A Matlab Toolbox to Solve Dynamic Multivariate RI Problems in the LQG Framework, working paper, Boston University
% Miao, Jianjun, Jieran Wu, and Eric Young, 2020, Multivariate Rational Inattention, working paper, Boston University
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [X_full, u_full, X, u, SigxJ, SiguJ, Sigx_fullJ,Sigu_fullJ] = RI_IRF3(A,B,F,L, Period, Sigma, C, V, J)

nx = size(A,1); %State dimension
nu = size(B,2); %Control dimension
ne = size(L,2); %Number of independent structural innovations
W = L*L';  %State noise covariance matrix
       
        r = size(C,1); %Signal dimension

        % Kalman gain matrix and auxilary matrices
        K = (A*Sigma*A'+W)*C'*pinv(C*(A*Sigma*A'+W)*C'+V);
        KC = K*C;
        G = [eye(nx), zeros(nx,nx); -KC, eye(nx)]; % By construction, G is invertible
        Ab = [A, -B*F; zeros(nx,nx), (eye(nx)-KC)*(A-B*F)]; 
        N = [L,zeros(nx,r);zeros(nx,ne),K*sqrtm(V)]; % 2n by ne+r matrix

    % Construct augmented state space representation

    Aa = inv(G)*Ab;
    Va = inv(G)*N; 

   
    % Compute IRFs under partial information
    X = zeros(2*nx,Period,(ne+r)); % First nx dimension in X are the response of true state
    u = zeros(nu,Period,(ne+r));
    
    I = eye(ne+r); % possible innovation matrix

    for i = 1:ne+r     
    
        X(:,1,i) = Va*I(:,i); % One-standard-deviation innovation shock 
        u(:,1,i) = -F*X(nx+1:end,1,i);  % initial response of optimal control is function of estimated states
    
        for j = 2:Period
        
        X(:,j,i)= Aa*X(:,j-1,i);
        u(:,j,i) = -F*X(nx+1:end,j,i);
         
        end

    end
    
    %% Compute IRFs under full information
    
     X_full = zeros(nx,Period,ne); % True state dynamics under full information, 
                                         %in pure tracking problem equivalent to first part of X
     u_full = zeros(nu,Period,ne); % control or endogenous decision under full information

    I1 = eye(ne); % possible innovation matrix under full information

    for i = 1:ne
        X_full(:,1,i) = L*I1(:,i);
        u_full(:,1,i) = -F*X_full(:,1,i);
    
        for j = 2:Period
            X_full(:,j,i) = (A-B*F)* X_full(:,j-1,i);  % Include B = 0, pure tracking case
            u_full(:,j,i) = -F*X_full(:,j,i);
        end
    end
    
     if nargout >4 %Generate second moments of order J
    %Doubling algorithm to compute moments under partial information
     Aa_old = Aa;
     Sig_old = Va*Va';
     SigX_old= eye(size(Aa));
     Diff = 0.1;
     while Diff >1e-25;
     SigX = Aa_old*SigX_old*Aa_old' + Sig_old; %Update
     Diff = max(max(abs(SigX-SigX_old)));
     Sig_old = Aa_old*Sig_old*Aa_old' + Sig_old;
     Aa_old = Aa_old*Aa_old;
     SigX_old = SigX;
    end    %while Diff       
   
    %Compute E{X(t)*X(t+J)'}
    SigXJ = Aa^(-min(0,J))*SigX*(Aa')^(max(0,J));
    SigxJ = SigXJ(1:nx,1:nx); % E[x(t)x(t+J)']
    SigxhatJ = SigXJ(nx+1:end,nx+1:end); %E[\hat{x}(t)\hat{x}(t+J)']       
    SiguJ = F*SigxhatJ*F';  %Compute E{u(t)*u(t+J)'} under RI
   
    %Doubling algorithm to compute moments under Full Information
     Ab_old = A-B*F;
     Sig_old = L*L';
     SigX_old= eye(size(Ab_old));
     Diff = 0.1;
     while Diff >1e-25;
     SigX = Ab_old*SigX_old*Ab_old' + Sig_old; %Update
     Diff = max(max(abs(SigX-SigX_old)));
     Sig_old = Ab_old*Sig_old*Ab_old' + Sig_old;
     Ab_old = Ab_old*Ab_old;
     SigX_old = SigX;
    end    %while Diff       
     Sigx_fullJ = Ab_old^(-min(0,J))*SigX*(Ab_old')^(max(0,J));
     Sigu_fullJ = F*Sigx_fullJ*F';
     
     end 

end

